"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DomainEventEmitter = void 0;
const node_core_1 = require("@lokalise/node-core");
const HandlerSpy_1 = require("../queues/HandlerSpy");
const node_crypto_1 = require("node:crypto");
class DomainEventEmitter {
    eventRegistry;
    metadataFiller;
    logger;
    errorReporter;
    transactionObservabilityManager;
    _handlerSpy;
    eventHandlerMap;
    constructor(deps, options = {}) {
        this.eventRegistry = deps.eventRegistry;
        this.metadataFiller = deps.metadataFiller;
        this.logger = deps.logger;
        this.errorReporter = deps.errorReporter;
        this.transactionObservabilityManager = deps.transactionObservabilityManager;
        this._handlerSpy =
            (0, HandlerSpy_1.resolveHandlerSpy)(options);
        this.eventHandlerMap = {};
    }
    get handlerSpy() {
        if (!this._handlerSpy) {
            throw new Error('HandlerSpy was not instantiated, please pass `handlerSpy` parameter during queue service creation.');
        }
        return this._handlerSpy;
    }
    async emit(supportedEvent, data, precedingMessageMetadata) {
        const eventTypeName = supportedEvent.publisherSchema.shape.type.value;
        if (!this.eventRegistry.isSupportedEvent(eventTypeName)) {
            throw new node_core_1.InternalError({
                errorCode: 'UNKNOWN_EVENT',
                message: `Unknown event ${eventTypeName}`,
            });
        }
        if (!data.timestamp)
            data.timestamp = this.metadataFiller.produceTimestamp();
        if (!data.id)
            data.id = this.metadataFiller.produceId();
        if (!data.metadata) {
            data.metadata = this.metadataFiller.produceMetadata(
            // @ts-ignore
            data, supportedEvent, precedingMessageMetadata ?? {});
        }
        if (!data.metadata.correlationId)
            data.metadata.correlationId = this.metadataFiller.produceId();
        const validatedEvent = this.eventRegistry
            .getEventDefinitionByTypeName(eventTypeName)
            .publisherSchema.parse({ type: eventTypeName, ...data });
        await this.handleEvent(validatedEvent);
        if (this._handlerSpy) {
            this._handlerSpy.addProcessedMessage({
                // @ts-ignore
                message: validatedEvent,
                processingResult: 'consumed',
            }, validatedEvent.id);
        }
        // @ts-ignore
        return validatedEvent;
    }
    /**
     * Register handler for a specific event
     */
    on(eventTypeName, handler, isBackgroundHandler = false) {
        if (!this.eventHandlerMap[eventTypeName]) {
            this.eventHandlerMap[eventTypeName] = { foreground: [], background: [] };
        }
        if (isBackgroundHandler)
            this.eventHandlerMap[eventTypeName].background.push(handler);
        else
            this.eventHandlerMap[eventTypeName].foreground.push(handler);
    }
    /**
     * Register handler for multiple events
     */
    onMany(eventTypeNames, handler, isBackgroundHandler = false) {
        for (const eventTypeName of eventTypeNames) {
            this.on(eventTypeName, handler, isBackgroundHandler);
        }
    }
    /**
     * Register handler for all events supported by the emitter
     */
    onAny(handler, isBackgroundHandler = false) {
        for (const supportedEvent of this.eventRegistry.supportedEvents) {
            this.on(supportedEvent.consumerSchema.shape.type.value, handler, isBackgroundHandler);
        }
    }
    async handleEvent(event) {
        const eventHandlers = this.eventHandlerMap[event.type] ?? {
            foreground: [],
            background: [],
        };
        for (const handler of eventHandlers.foreground) {
            const transactionId = (0, node_crypto_1.randomUUID)();
            let isSuccessfull = false;
            try {
                this.transactionObservabilityManager?.startWithGroup(this.buildTransactionKey(event, handler, false), transactionId, event.type);
                await handler.handleEvent(event);
                isSuccessfull = true;
            }
            finally {
                this.transactionObservabilityManager?.stop(transactionId, isSuccessfull);
            }
        }
        for (const handler of eventHandlers.background) {
            const transactionId = (0, node_crypto_1.randomUUID)();
            this.transactionObservabilityManager?.startWithGroup(this.buildTransactionKey(event, handler, true), transactionId, event.type);
            Promise.resolve(handler.handleEvent(event))
                .then(() => {
                this.transactionObservabilityManager?.stop(transactionId, true);
            })
                .catch((error) => {
                this.transactionObservabilityManager?.stop(transactionId, false);
                const context = {
                    event: JSON.stringify(event),
                    eventHandlerId: handler.eventHandlerId,
                    'x-request-id': event.metadata?.correlationId,
                };
                this.logger.error({
                    ...(0, node_core_1.resolveGlobalErrorLogObject)(error),
                    ...context,
                });
                this.errorReporter?.report({ error: error, context });
            });
        }
    }
    buildTransactionKey(event, handler, isBackgroundHandler) {
        return `${isBackgroundHandler ? 'bg' : 'fg'}_event_listener:${event.type}:${handler.eventHandlerId}`;
    }
}
exports.DomainEventEmitter = DomainEventEmitter;
//# sourceMappingURL=DomainEventEmitter.js.map